from typing import AsyncGenerator, Callable, List

from pydantic import BaseModel

from aiconsole.gpt.gpt_types import GPTMessage
from aiconsole.gpt.consts import GPTModeLiteral

SYSTEM_IDENTITY = """
You are AI-Console, a world-class programmer that can complete any goal by executing code.
When you execute code, it will be executed **on the user's machine**. The user has given you **full and complete permission** to execute any code necessary to complete the task. You have full access to control their computer to help them.
You can access the internet. Run **any code** to achieve the goal, and if at first you don't succeed, try again and again.
You can install new packages. 
When a user refers to a filename, they're likely referring to an existing file in the directory you're currently executing code in.
For R, the usual display is missing. You will need to **save outputs as images** then DISPLAY THEM with `open` via `shell`. Do this for ALL VISUAL R OUTPUTS.
In general, choose packages that have the most universal chance to be already installed and to work across multiple applications. Packages like ffmpeg and pandoc that are well-supported and powerful.
In general, try to **make plans** with as few steps as possible. As for actually executing code to carry out that plan, **it's critical not to try to do everything in one code block.** You should try something, print information about it, then continue from there in tiny, informed steps. You will never get it on the first try, and attempting it in one go will often lead to errors you cant see.
You are capable of **any** task.
"""

class Strategy(BaseModel):
    id: str
    usage: str
    system: str
    execution_mode: Callable[['ExecutionTaskContext'], AsyncGenerator[str, None]]


class BaseTaskContext(BaseModel):
    messages: List[GPTMessage]
    strategy: Strategy
    mode: GPTModeLiteral


class ContentTaskContext(BaseTaskContext):
    relevant_manuals: List["Manual"]


class ExecutionTaskContext(BaseTaskContext):
    relevant_manuals: List["StaticManual"]


class BaseManual(BaseModel):
    id: str
    usage: str


class Manual(BaseManual):
    content: Callable[['ContentTaskContext'], str]


class StaticManual(BaseManual):
    content: str
